; ************************************************************************
; Resonant Low-Pass Filter
;*************************************************************************

sr = 44100
kr  =  44100
ksmps =1
nchnls = 2

;******************************************************************
instr 1

  idur  =      p3
  iamp  =      p4
  kstep init   0

  loop1:
        timout 0, p5, cont1
        kstep = frac((kstep + 1)/8)*8
        reinit loop1
  cont1:

  kpch1 table  kstep, 20
  kfqc  =      cpspch(kpch1)
  axn   oscil  iamp, kfqc, 10
  outs axn, axn

endin


instr 15   ; Table Based Rezzy Synth with Enveloped Resonance

idur     =    p3
iamp     =    p4
ifqc     =    cpspch(p5)
irez     =    p7
itabl1   =    p8
kvalley  init 0
kpeak    init 0
knewval  init 1
knewpk   init 1
ksaveval init 0
ksavepk  init 0

; Initialize Yn-1 & Yn-2 to zero
kynm1      init 0
kynm2      init 0
ktynm1     init 0
ktynm2     init 0
kfrangenm1 init 0
kfrangenm2 init 0
knewrez    init 0
kfsweep    init 30
kstep      init 0
knxtstp    init 1

; Frequency Sweep
  kfsweep expseg 30, .5*p3, p6, .5*p3, 30

; Sequencer Section
  loop1:

      kdur  table  kstep, 21
      kdur1  =     kdur/8

;     Frequency Envelope
;      kfco expseg 20, .1, i(kfsweep), p5-.1, 20
      kfco expseg 20, .1, i(kfsweep), i(kdur1)-.1, 20

;     Amplitude envelope
;      kaenv  linseg 0, .01, 1, p5-.02, 1, .01, 0
      kaenv  linseg 0, .01, 1, i(kdur1)-.02, 1, .01, 0

;     Panning
      kpanleft  table kstep, 22
      kpanright table kstep, 23

      kpch1 table  kstep, 20
      kfqc1 =      cpspch(kpch1)
      kpch2 table  knxtstp, 20
      kfqc2 =      cpspch(kpch2)
      if (p9 = 0) goto skipslide
        kfqc  linseg i(kfqc1), i(kdur1)-.04, i(kfqc1), .04, i(kfqc2)
        goto nxtslide

      skipslide:
        kfqc  = kfqc1

      nxtslide:

;      timout 0, p5, cont1
      timout 0, i(kdur1), cont1
        kstep   = frac((kstep + 1)/8)*8
        knxtstp = frac((kstep + 1)/8)*8

        reinit loop1
  cont1:

; This relationship attempts to separate Freq from Res.
    ka1    = 100/irez/sqrt(kfco)-1
    kta1   = 100/sqrt(kfco)-1
    ka2    = 1000/kfco


; Oscillator
  kxn oscil iamp, kfqc, itabl1
;  kxn2 oscil 1, kfqc/2, 2

;  kxn  =     kxn1*kxn2;+iamp*kxn2/2


; Replace the differential eq. with a difference eq.
  kyn = ((ka1+2*ka2)*kynm1-ka2*kynm2+kxn)/(1+ka1+ka2)
  ktyn = ((kta1+2*ka2)*ktynm1-ka2*ktynm2+kxn)/(1+kta1+ka2)

; This section determines current peak and valley values and the range.
; As long as the signal rises it updates peak.  When if falls it updates valley.

  koldrez = knewrez
  knewrez = kyn-ktyn

  if (koldrez>=knewrez) goto next1
    if (knewval != 1) goto next2
      knewval = 0
      knewpk  = 1
      kvalley = ksaveval
next2:
    ksavepk = knewrez

next1:
  if (koldrez<=knewrez) goto next 3
    if (knewpk != 1) goto next4
      knewpk  = 0
      knewval = 1
      kpeak   = ksavepk
next4:
    ksaveval = knewrez

next3:
; The current range is calculated and low pass filtered.

  krangen = abs(kpeak-kvalley)

  kfrangen = ((kta1+2*ka2)*kfrangenm1-ka2*kfrangenm2+krangen)/(1+kta1+ka2)

; The filtered range is used to envelope the resonance then distortion is added.
  kclip1 = knewrez*kfrangen/irez/120000000
  kclip tablei kclip1, 7, 1, .5
  kout = ktyn + kclip*40000

;  kout = ktyn+knewrez*kfrangen/irez/100

; Update everything for the next pass.
  kynm2 = kynm1
  kynm1 = kyn
  ktynm2 = ktynm1
  ktynm1 = ktyn
  kfrangenm2 = kfrangenm1
  kfrangenm1 = kfrangen

  aout = kout
;  axn  = kxn
;  axn2  = kxn2

; Amp envelope and output
  outs kaenv*kpanleft*aout, kaenv*kpanright*aout
;  outs kaenv*axn, kaenv*axn2*10000

endin

