;-------------------------------------------------------------
; Physical Models
; coded:        1/22/97 Hans Mikelson
;-------------------------------------------------------------
sr       =      44100
kr       =      4410
ksmps    =      10
nchnls   =      1

;-------------------------------------------------------------
;Bass Physical Model
;-------------------------------------------------------------
         instr  2

; Initializations
;-------------------------------------------------------------
ifqc    init   cpspch(p5)
ipluck  init   1/ifqc*p6
kcount  init   0
adline  init   0
ablock2 init   0
ablock3 init   0
afiltr  init   0
afeedbk init   0

;-------------------------------------------------------------
koutenv linseg 0,.01,1,p3-.11,1,.1,0 ;Amplitude envelope
kfltenv linseg 0,1.5,1,1.5,0         ;Filter envelope

; This envelope loads the string with a triangle wave.
kenvstr linseg 0,ipluck/4,-p4/2,ipluck/2,p4/2,ipluck/4,0,p3-ipluck,0

;-------------------------------------------------------------
aenvstr =      kenvstr        ;Convert to audio rate for filtering
ainput  tone   aenvstr,200    ;Filter the trianle to round off the peak

ablock2 =      afeedbk-ablock3+.99*ablock2  ;This is a DC blocking filter
ablock3 =      afeedbk                      ;used to prevent drift away from
ablock  =      ablock2                      ;zero.

;-------------------------------------------------------------
adline  delay ablock+ainput,1/ifqc-15/sr  ;Delay line length 1/frequency.
afeedbk tone  adline,400                  ;Filtered feedback.

;-------------------------------------------------------------
abody1  reson afeedbk,110,40            ;Two body resonances are added
abody1  =     abody1/5000                ;to the tone.
abody2  reson afeedbk,70,20
abody2  =     abody2/50000

;-------------------------------------------------------------
aout    =     afeedbk
        out   50*koutenv*(aout+kfltenv*(abody1+abody2))

        endin

;-------------------------------------------------------------
;Flute Instrument based on Perry Cook's Slide Flute
;-------------------------------------------------------------

         instr  3

aflute1  init   0
ifqc     =      cpspch(p5)
ipress   =      p6
ibreath  =      p7
ifeedbk1 =      p8
ifeedbk2 =      p9

;-------------------------------------------------------------
kenv1    linseg 0,.06,1.1*ipress,.2,ipress,p3-.16,ipress,.02,0 ;Flow setup
kenv2    linseg 0,.01,1,p3-.02,1,.01,0     ;Flow must be about 1 or it will blow up
kenvibr  linseg 0,.5,0,.5,1,p3-1,1         ; Vibrato envelope

;-------------------------------------------------------------
aflow1   rand   kenv1          ;Noise is used to simulate breath sound.
kvibr    oscil  .1*kenvibr,5,3              ;Low frequency vibrato
asum1    =      ibreath*aflow1+kenv1+kvibr  ;Add flow, noise and vibrato.
asum2    =      asum1+aflute1*ifeedbk1      ;Add above to scaled feedback.
afqc     =      1/ifqc-asum1/20000-9/sr+ifqc/12000000 ;Find delay length.

;-------------------------------------------------------------
atemp1   delayr   1/ifqc/2                  ;The embouchoure delay should
ax       deltapi  afqc/2                    ;be about 1/2 the bore delay.
         delayw   asum2

;-------------------------------------------------------------
apoly    =        ax-ax*ax*ax               ;A polynomial is used to adjust
asum3    =        apoly+aflute1*ifeedbk2    ;the feedback.

avalue   tone     asum3,2000

; Bore, the bore length determines pitch.  Shorter is higher pitch.
;-------------------------------------------------------------
atemp2   delayr   1/ifqc
aflute1  deltapi  afqc
         delayw   avalue

         out      avalue*p4*kenv2

         endin

;-------------------------------------------------------------
; Clarinet Instrument based on Perry Cook's Clarinet
;-------------------------------------------------------------

          instr    4
areedbell init     0
ifqc      =        cpspch(p5)
ifco      =        p7
ibore     =        1/ifqc-15/sr

kenv1     linseg   0,.005,.55+.3*p6,p3-.015,.55+.3*p6,.01,0
kenvibr   linseg   0,.1,0,.9,1,p3-1,1  ;Vibrato envelope

kemboff   =        p8                  ;Can be used to adjust reed stiffness.

avibr oscil .1*kenvibr, 5, 3           ;Breath pressure.
apressm = kenv1 + avibr

arefilt tone areedbell, ifco  ; Reflection filter from the bell is lowpass.

abellreed delay arefilt, ibore  ; The delay from bell to reed.

; Back pressure and reed table look up.
  asum2 = - apressm -.95*arefilt - kemboff
  areedtab tablei asum2/4+.34, p9, 1, .5
  amult1 = asum2 * areedtab

; Forward Pressure
  asum1 = apressm + amult1

  areedbell delay asum1, ibore

  aofilt atone areedbell, ifco

  out aofilt*p4

endin

instr 8  ; Drum Stick 1

  gadrum init 0

; Frequency
  ifqc  = cpspch(p5)

; Amp Envelope
;kampenv linseg 0,.01,1,p3-.01,1

; Initialize the delay line with noise
  ashape linseg 0, 1/ifqc/8, -p4/2, 1/ifqc/4, p4/2, 1/ifqc/8, 0, p3-1/ifqc, 0
  gadrum tone ashape, p6

endin

instr 9  ; A square drum

  itube    init p7
  ifdbck1  init p8
  ifdbck2  init p9
  iwidth   init p10
  ilength  init p11
  idiagnl  = sqrt(iwidth*iwidth+ilength*ilength)

  anodea   init 0
  anodeb   init 0
  anodec   init 0
  anoded   init 0
  afiltr   init 0
  ablocka2 init 0
  ablocka3 init 0
  ablockb2 init 0
  ablockb3 init 0
  ablockc2 init 0
  ablockc3 init 0
  ablockd2 init 0
  ablockd3 init 0

; Frequency
  ifqc  = cpspch(p5)
  ipfilt = p6

; Amplitude Envelope
  kampenv   linseg 0, .01, p4, p3-.02, p4, .01, 0

; Delay Lines
  alineab delay anodea+gadrum+afiltr, ilength/ifqc
  alineba delay anodeb+gadrum+afiltr, ilength/ifqc
  alinebc delay anodeb+gadrum+afiltr, iwidth/ifqc
  alinecb delay anodec+gadrum+afiltr, iwidth/ifqc
  alinecd delay anodec+gadrum+afiltr, ilength/ifqc
  alinedc delay anoded+gadrum+afiltr, ilength/ifqc
  alinead delay anodea+gadrum+afiltr, iwidth/ifqc
  alineda delay anoded+gadrum+afiltr, iwidth/ifqc
  alineac delay anodea+gadrum+afiltr, idiagnl/ifqc
  alineca delay anodec+gadrum+afiltr, idiagnl/ifqc
  alinebd delay anodeb+gadrum+afiltr, idiagnl/ifqc
  alinedb delay anoded+gadrum+afiltr, idiagnl/ifqc

; Filter the delayed signal and feedback into the delay.
; Implements DC blocking.
  ablocka1 = -(alineba + alineca + alineda)/ifdbck1
  ablocka2 = ablocka1 - ablocka3 + .99*ablocka2
  ablocka3 = ablocka1
  anodea   = ablocka2

; Node B
  ablockb1 = -(alineba + alineca + alineda)/ifdbck1
  ablockb2 = ablockb1 - ablockb3 + .99*ablockb2
  ablockb3 = ablockb1
  anodeb   = ablockb2

; Node C
  ablockc1 = -(alineba + alineca + alineda)/ifdbck1
  ablockc2 = ablockc1 - ablockc3 + .99*ablockc2
  ablockc3 = ablockc1
  anodec   = ablockc2

; Node D
  ablockd1 = -(alineba + alineca + alineda)/ifdbck1
  ablockd2 = ablockd1 - ablockd3 + .99*ablockd2
  ablockd3 = ablockd1
  anoded   = ablockd2

; Body resonance
  atube  delay anodea, itube/ifqc
  afiltr tone  atube, 1000
  afiltr = afiltr/ifdbck2

; Scale and output
  out anodea*kampenv*1000

endin

