sr     =     44100                   ;These instruments require sr=kr
kr     =     44100
ksmps  =     1
nchnls =     2

;-------------------------------------------------------------------------
; Saw-Ramp Transform Rezzy Synth
;-------------------------------------------------------------------------
       instr 1   

idur   init  p3
iamp   init  p4
ifqc   init  cpspch(p5)                   ;Convert to cycles/second
irez   init  p7 
aynm1  init  0                            ;Initialize Yn-1
aynm2  init  0                            ;& Yn-2 to zero.
kx     init  -p4
kdx    init  p4*4*ifqc/sr
kmax   init  p4

klfo   oscil .9,1,1                     ;Low Frequency Oscillator
klfo   =     klfo+1                     ;for triangle ramp transform.
kdx1   =     4*p4*ifqc/sr/klfo
kdx2   =     2*p4*ifqc/(sr-sr/2*klfo)

;-------------------------------------------------------------------------
kaenv  linseg 0,.01,1,p3-.02,1,.01,0      ;Amplitude envelope
kfco   expseg 10,.2*p3,p6,.4*p3,.5*p6,.4*p3,.1*p6  ;Frequency sweep

;-------------------------------------------------------------------------
ka1    =      100/irez/sqrt(kfco)-1       ;Adjust Q and Fco for
ka2    =      1000/kfco                   ;coefficients A1 & A2.

;-------------------------------------------------------------------------

;Bouncing ball algorithm-a ball bouncing between two parallel walls ie:Pong
;-------------------------------------------------------------------------
knewx=kx+kdx                            ;Find next position 

if (knewx<=kmax) goto next1             ;If next is beyond the top wall
  knewx=kmax-kdx2*(kdx1-kmax+kx)/kdx1   ;then find the bounce position
  kdx=-kdx2                             ;and downward slope.

next1:
if (knewx>=-kmax) goto next2            ;If next is beyond the bottom wall
  knewx=-kmax+kdx1*(kdx2-kmax-kx)/kdx2  ;then find the bounce position
  kdx=kdx1                              ;and upward slope.

next2:
kx=knewx

axn  =  kx

;-------------------------------------------------------------------------
ayn    =     ((ka1+2*ka2)*aynm1-ka2*aynm2+axn)/(1+ka1+ka2) ;Difference eq. app.
aynm2  =     aynm1                                        ;of differential eq.
aynm1  =     ayn                                          ;Save Yn-1 & Yn-2.

;-------------------------------------------------------------------------
aout   =     ayn*kaenv                      ;Scale and output
       outs   aout,axn

       endin


;-------------------------------------------------------------------------
; Oscillator Sync Rezzy Synth
;-------------------------------------------------------------------------
       instr 2   

idur   init  p3
iamp   init  p4
ifqc   init  cpspch(p5)                   ;Convert to cycles/second
ifqcs  init  cpspch(p5)*p8
irez   init  p7 
aynm1  init  0                            ;Initialize Yn-1
aynm2  init  0                            ;& Yn-2 to zero.
kxn    init  -p4
kdx    init  p4*8*ifqc/sr
kmax   init  p4
ksyncm init -1
ksyncs init -1

;-------------------------------------------------------------------------
kaenv  linseg 0,.001,1,p3-.002,1,.001,0      ;Amplitude envelope
kfco   expseg 10,.2*p3,p6,.4*p3,.5*p6,.4*p3,.1*p6  ;Frequency sweep

;-------------------------------------------------------------------------
ka1    =      100/irez/sqrt(kfco)-1       ;Adjust Q and Fco for
ka2    =      1000/kfco                   ;coefficients A1 & A2.

;-------------------------------------------------------------------------
kxn=kxn+kdx                            ;Find next position 

ksmprev =     ksyncm                   ;Watch master for transition
ksyncm  oscil 1,ifqc,2                 ;from -1 to 1 then reset axn.
kxn = ((ksmprev=-1)&&(ksyncm=1) ? -kmax : kxn)

kssprev =     ksyncs                   ;Watch slave for transition
ksyncs  oscil 1,ifqcs,2                ;from -1 to 1 then reset axn.
kxn = ((kssprev=-1)&&(ksyncs=1) ? -kmax : kxn)

axn=kxn
;-------------------------------------------------------------------------
ayn    =     ((ka1+2*ka2)*aynm1-ka2*aynm2+axn)/(1+ka1+ka2) ;Difference eq. app.
aynm2  =     aynm1                                        ;of differential eq.
aynm1  =     ayn                                          ;Save Yn-1 & Yn-2.

;-------------------------------------------------------------------------
aout   =     ayn*kaenv                      ;Scale and output
       outs   aout,axn

       endin

;-------------------------------------------------------------------------
; Bifurcation Rezzy Synth
;-------------------------------------------------------------------------
       instr 3   

idur   init  p3
iamp   init  p4
ifqc   init  cpspch(p5)                   ;Convert to cycles/second
irez   init  p7 
aynm1  init  0                            ;Initialize Yn-1
aynm2  init  0                            ;& Yn-2 to zero.
kxn    init  p8
ksyncm init -1
ib     init p9


;-------------------------------------------------------------------------
kaenv  linseg 0,.001,1,p3-.002,1,.001,0      ;Amplitude envelope
kfco   expseg 10,.2*p3,p6,.4*p3,.5*p6,.4*p3,.1*p6  ;Frequency sweep

;-------------------------------------------------------------------------
ka1    =      100/irez/sqrt(kfco)-1       ;Adjust Q and Fco for
ka2    =      1000/kfco                   ;coefficients A1 & A2. 

;-------------------------------------------------------------------------

ksmprev =     ksyncm                   ;Watch master for transition
ksyncm  oscil 1,ifqc,2                 ;from -1 to 1 then reset kxn.
kxn  = ((ksmprev=-1)&&(ksyncm=1) ? (kxn * ib*(1 - kxn)) : kxn)

axn=kxn
;-------------------------------------------------------------------------
ayn    =     ((ka1+2*ka2)*aynm1-ka2*aynm2+axn)/(1+ka1+ka2) ;Difference eq. app.
aynm2  =     aynm1                                        ;of differential eq.
aynm1  =     ayn                                          ;Save Yn-1 & Yn-2.

;-------------------------------------------------------------------------
aout1   =     ayn*kaenv*iamp*4                      ;Scale and output
aout   butterhp aout1, 10
       outs   aout,aout

       endin