sr=44100
kr=22050
ksmps=2
nchnls=2
zakinit 30,30

instr 1 ;Global initialization

  gi2fqc init cpspch(p4)
  gi3fqc init cpspch(p5)

endin

instr 2 ; Epicycloid or Spirograph curve
        ; This set of parametric equations defines the path traced by
        ; a point on a circle of radius B rotating outside a circle of
        ; radius A.

ifqc   init cpspch(p5)
ifqci  init gi2fqc
gi2fqc init ifqc
ia     init p6
ib     init p7
ihole  init p8
iscale init 1/(ia+2*ib)

kampenv linseg 0, .02, p4*iscale, p3-.04, p4*iscale, .02, 0
kptchenv linseg ifqci, .2*p3, ifqc, .8*p3, ifqc
kvibenv  linseg 0, .5, 0, .2, 1, .2, 1
kvibr oscil 20, 8, 1
kfqc = kptchenv+kvibr*kvibenv

; Sine and Cosine
acos1 oscil ia+ib, kfqc, 1, .25
acos2 oscil ib*ihole, (ia-ib)/ib*kfqc, 1, .25
ax = acos1 + acos2

asin1 oscil ia+ib, kfqc, 1
asin2 oscil ib, (ia-ib)/ib*kfqc, 1
ay = asin1 - asin2

outs kampenv*ax, kampenv*ay

endin

instr 3 ; Hypocycloid or Spirograph curve
        ; This set of parametric equations defines the path traced by
        ; a point on a circle of radius B rotating inside a circle of
        ; radius A.

ifqc = cpspch(p5)
ifqci  init gi3fqc
gi3fqc init ifqc
ia   = p6
ib   = p7
ihole = p8
iscale = (ia<ib ? 1/ib:1/ia)

kampenv linseg 0, .1, p4*iscale, p3-.2, p4*iscale, .1, 0
kptchenv linseg ifqci, .2*p3, ifqc, .8*p3, ifqc
kvibenv  linseg 0, .5, 0, .2, 1, .2, 1
kvibr oscil 20, 8, 1
kfqc = kptchenv+kvibr*kvibenv

; Sine and Cosine
acos1 oscil ia-ib, kfqc, 1, .25
acos2 oscil ib*ihole, (ia-ib)/ib*kfqc, 1, .25
ax = acos1 + acos2

asin1 oscil ia-ib, kfqc, 1
asin2 oscil ib, (ia-ib)/ib*kfqc, 1
ay = asin1 - asin2

outs kampenv*ax, kampenv*ay

endin

instr 4 ; Banchoff Klein Bottle

ifqc    init cpspch(p5)
iu      init p6
iv      init p7
irt2 init sqrt(2)

kampenv linseg 0, .02, p4, p3-.04, p4, .02, 0

; Cosines
acosu  oscil 1, iu*ifqc,   1, .25
acosu2 oscil 1, iu*ifqc/2, 1, .25
acosv  oscil 1, iv*ifqc,   1, .25

; Sines
asinu  oscil 1, iu*ifqc,   1
asinu2 oscil 1, iu*ifqc/2, 1
asinv  oscil 1, iv*ifqc,   1

; Compute X and Y
ax = acosu*(acosu2*(irt2+acosv)+asinu2*asinv*acosv)
ay = asinu*(acosu2*(irt2+acosv)+asinu2*asinv*acosv)

; Low frequency rotation in spherical coordinates z, phi, theta.
  klfsinth oscil 1,4,1
  klfsinph oscil 1,1,1
  klfcosth oscil 1,4,1,.25
  klfcosph oscil 1,1,1,.25

  aox=-ax*klfsinth+ay*klfcosth
  aoy=-ax*klfsinth*klfcosph-ay*klfsinth*klfcosph+klfsinph

outs kampenv*aox, kampenv*aoy

endin

; Spherical Lissajous Figures
        instr 5

ifqc    init cpspch(p5)
iu      init p6
iv      init p7
irt2    init sqrt(2)
iradius init 1
ioutx   init p8
iouty   init p9
ioutz   init p10

kampenv linseg 0, .001, p4, p3-.002, p4, .001, 0

; Cosines
acosu  oscil 1, iu*ifqc,   1, .25
acosv  oscil 1, iv*ifqc,   1, .25

; Sines
asinu  oscil 1, iu*ifqc,   1
asinv  oscil 1, iv*ifqc,   1

; Compute X and Y
ax = iradius*asinu*acosv
ay = iradius*asinu*asinv
az = iradius*acosu

   zawm kampenv*ax, ioutx
   zawm kampenv*ay, iouty
   zawm kampenv*az, ioutz

endin

        instr 50

ifqc    =    p4
iphase  =    p5
iplane  =    p6
inx     =    p7
iny     =    p8
inz     =    p9
ioutx   =    p10
iouty   =    p11
ioutz   =    p12

kcost  oscil 1, ifqc,   1, .25+iphase
ksint  oscil 1, ifqc,   1, iphase

ax      zar  inx
ay      zar  iny
az      zar  inz

; Rotation in X-Y plane
  if (iplane!=1) goto next1
    axr = ax*kcost + ay*ksint
    ayr =-ax*ksint + ay*kcost
    azr = az
    goto next3

; Rotation in X-Z plane
next1:
  if (iplane!=2) goto next2
    axr = ax*kcost + az*ksint
    ayr = ay
    azr =-ax*ksint + az*kcost
    goto next 3

; Rotation in Y-Z plane
next2:
    axr = ax
    ayr = ay*kcost + az*ksint
    azr =-ay*ksint + az*kcost

next3:
        zaw axr, ioutx
        zaw ayr, iouty
        zaw azr, ioutz

endin

; Mixer
instr 100

in1  =  p4
in2  =  p5

a1   zar  in1
a2   zar  in2

     outs a1, a2

     zacl 0,30
endin
